@@ -16,9 +16,10 @@ from PyQt5.Qt import (
     QPainter, QStaticText, pyqtSignal, QTextOption, QAbstractListModel,
     QModelIndex, QStyledItemDelegate, QStyle, QCheckBox, QListView,
     QTextDocument, QSize, QComboBox, QFrame, QCursor, QGroupBox, QSplitter,
-    QPixmap, QRect, QPlainTextEdit, pyqtSlot, QMimeData, QKeySequence)
+    QPixmap, QRect, QPlainTextEdit, QMimeData)
 
 from calibre import prepare_string_for_xml, human_readable
+from calibre.constants import iswindows
 from calibre.ebooks.oeb.polish.utils import lead_text, guess_type
 from calibre.gui2 import error_dialog, choose_files, choose_save_file, info_dialog, choose_images
 from calibre.gui2.tweak_book import tprefs, current_container
@@ -30,6 +31,7 @@ from calibre.gui2.complete2 import EditWithComplete
 ROOT = QModelIndex()
 PARAGRAPH_SEPARATOR = '\u2029'
 
+
 class BusyCursor(object):
 
     def __enter__(self):
@@ -38,11 +40,13 @@ class BusyCursor(object):
     def __exit__(self, *args):
         QApplication.restoreOverrideCursor()
 
+
 class Dialog(BaseDialog):
 
     def __init__(self, title, name, parent=None):
         BaseDialog.__init__(self, title, name, parent=parent, prefs=tprefs)
 
+
 class InsertTag(Dialog):  # {{{
 
     def __init__(self, parent=None):
@@ -76,6 +80,7 @@ class InsertTag(Dialog):  # {{{
 
 # }}}
 
+
 class RationalizeFolders(Dialog):  # {{{
 
     TYPE_MAP = (
@@ -117,7 +122,7 @@ class RationalizeFolders(Dialog):  # {{{
             l.addWidget(le, i + 1, 1)
         self.la2 = la = QLabel(_(
             'Note that this will only arrange files inside the book,'
-            ' it will not affect how they are displayed in the Files Browser'))
+            ' it will not affect how they are displayed in the File browser'))
         la.setWordWrap(True)
         l.addWidget(la, i + 2, 0, 1, -1)
         l.addWidget(self.bb, i + 3, 0, 1, -1)
@@ -135,6 +140,7 @@ class RationalizeFolders(Dialog):  # {{{
         return Dialog.accept(self)
 # }}}
 
+
 class MultiSplit(Dialog):  # {{{
 
     def __init__(self, parent=None):
@@ -169,6 +175,7 @@ class MultiSplit(Dialog):  # {{{
 
 # }}}
 
+
 class ImportForeign(Dialog):  # {{{
 
     def __init__(self, parent=None):
@@ -251,6 +258,7 @@ class ImportForeign(Dialog):  # {{{
 
 # Quick Open {{{
 
+
 def make_highlighted_text(emph, text, positions):
     positions = sorted(set(positions) - {-1})
     if positions:
@@ -265,6 +273,7 @@ def make_highlighted_text(emph, text, positions):
         return ''.join(parts)
     return text
 
+
 class Results(QWidget):
 
     EMPH = "color:magenta; font-weight:bold"
@@ -395,6 +404,7 @@ class Results(QWidget):
         except IndexError:
             pass
 
+
 class QuickOpen(Dialog):
 
     def __init__(self, items, parent=None):
@@ -469,6 +479,7 @@ class QuickOpen(Dialog):
 
 # Filterable names list {{{
 
+
 class NamesDelegate(QStyledItemDelegate):
 
     def sizeHint(self, option, index):
@@ -508,6 +519,7 @@ class NamesDelegate(QStyledItemDelegate):
             doc.drawContents(painter)
         painter.restore()
 
+
 class NamesModel(QAbstractListModel):
 
     filtered = pyqtSignal(object)
@@ -552,6 +564,7 @@ class NamesModel(QAbstractListModel):
         except IndexError:
             pass
 
+
 def create_filterable_names_list(names, filter_text=None, parent=None, model=NamesModel):
     nl = QListView(parent)
     nl.m = m = model(names, parent=nl)
@@ -569,6 +582,7 @@ def create_filterable_names_list(names, filter_text=None, parent=None, model=Nam
 
 # Insert Link {{{
 
+
 class AnchorsModel(QAbstractListModel):
 
     filtered = pyqtSignal(object)
@@ -601,13 +615,14 @@ class AnchorsModel(QAbstractListModel):
         self.endResetModel()
         self.filtered.emit(not bool(query))
 
+
 class InsertLink(Dialog):
 
     def __init__(self, container, source_name, initial_text=None, parent=None):
         self.container = container
         self.source_name = source_name
         self.initial_text = initial_text
-        Dialog.__init__(self, _('Insert Hyperlink'), 'insert-hyperlink', parent=parent)
+        Dialog.__init__(self, _('Insert hyperlink'), 'insert-hyperlink', parent=parent)
         self.anchor_cache = {}
 
     def sizeHint(self):
@@ -642,6 +657,7 @@ class InsertLink(Dialog):
         h.addLayout(fnl), h.setStretch(1, 1)
 
         self.tl = tl = QFormLayout()
+        tl.setFieldGrowthPolicy(tl.AllNonFixedFieldsGrow)
         self.target = t = QLineEdit(self)
         t.setPlaceholderText(_('The destination (href) for the link'))
         tl.addRow(_('&Target:'), t)
@@ -670,6 +686,8 @@ class InsertLink(Dialog):
             ac = self.anchor_cache[name] = []
             for item in set(root.xpath('//*[@id]')) | set(root.xpath('//h:a[@name]', namespaces={'h':XHTML_NS})):
                 frag = item.get('id', None) or item.get('name')
+                if not frag:
+                    continue
                 text = lead_text(item, num_words=4)
                 ac.append((text, frag))
             ac.sort(key=lambda text_frag: primary_sort_key(text_frag[0]))
@@ -715,6 +733,7 @@ class InsertLink(Dialog):
 
 # Insert Semantics {{{
 
+
 class InsertSemantics(Dialog):
 
     def __init__(self, container, parent=None):
@@ -724,7 +743,7 @@ class InsertSemantics(Dialog):
             for item in container.opf_xpath('//opf:guide/opf:reference[@href and @type]')}
         self.final_type_map = self.original_type_map.copy()
         self.create_known_type_map()
-        Dialog.__init__(self, _('Set Semantics'), 'insert-semantics', parent=parent)
+        Dialog.__init__(self, _('Set semantics'), 'insert-semantics', parent=parent)
 
     def sizeHint(self):
         return QSize(800, 600)
@@ -743,9 +762,9 @@ class InsertSemantics(Dialog):
             'dedication': _('Dedication'),
             'epigraph': _('Epigraph'),
             'foreword': _('Foreword'),
-            'loi': _('List of Illustrations'),
-            'lot': _('List of Tables'),
-            'notes:': _('Notes'),
+            'loi': _('List of illustrations'),
+            'lot': _('List of tables'),
+            'notes': _('Notes'),
             'preface': _('Preface'),
             'text': _('Text'),
         }
@@ -907,11 +926,12 @@ class InsertSemantics(Dialog):
 
 # }}}
 
+
 class FilterCSS(Dialog):  # {{{
 
     def __init__(self, current_name=None, parent=None):
         self.current_name = current_name
-        Dialog.__init__(self, _('Filter Style Information'), 'filter-css', parent=parent)
+        Dialog.__init__(self, _('Filter style information'), 'filter-css', parent=parent)
 
     def setup_ui(self):
         from calibre.gui2.convert.look_and_feel_ui import Ui_Form
@@ -978,6 +998,7 @@ class FilterCSS(Dialog):  # {{{
 
 # Add Cover {{{
 
+
 class CoverView(QWidget):
 
     def __init__(self, parent=None):
@@ -1015,6 +1036,7 @@ class CoverView(QWidget):
     def sizeHint(self):
         return QSize(300, 400)
 
+
 class AddCover(Dialog):
 
     import_requested = pyqtSignal(object, object)
@@ -1113,14 +1135,14 @@ class AddCover(Dialog):
 
 # }}}
 
+
 class PlainTextEdit(QPlainTextEdit):  # {{{
 
     ''' A class that overrides some methods from QPlainTextEdit to fix handling
-    of the nbsp unicode character. '''
+    of the nbsp unicode character and AltGr input method on windows. '''
 
     def __init__(self, parent=None):
         QPlainTextEdit.__init__(self, parent)
-        self.selectionChanged.connect(self.selection_changed)
         self.syntax = None
 
     def toPlainText(self):
@@ -1136,22 +1158,6 @@ class PlainTextEdit(QPlainTextEdit):  # {{{
         # non BMP characters such as 0x1f431 are present.
         return ans.rstrip('\0')
 
-    @pyqtSlot()
-    def copy(self):
-        # Workaround Qt replacing nbsp with normal spaces on copy
-        c = self.textCursor()
-        if not c.hasSelection():
-            return
-        md = QMimeData()
-        md.setText(self.selected_text)
-        QApplication.clipboard().setMimeData(md)
-
-    @pyqtSlot()
-    def cut(self):
-        # Workaround Qt replacing nbsp with normal spaces on copy
-        self.copy()
-        self.textCursor().removeSelectedText()
-
     def selected_text_from_cursor(self, cursor):
         return unicodedata.normalize('NFC', unicode(cursor.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0'))
 
@@ -1159,22 +1165,38 @@ class PlainTextEdit(QPlainTextEdit):  # {{{
     def selected_text(self):
         return self.selected_text_from_cursor(self.textCursor())
 
-    def selection_changed(self):
-        # Workaround Qt replacing nbsp with normal spaces on copy
-        clipboard = QApplication.clipboard()
-        if clipboard.supportsSelection() and self.textCursor().hasSelection():
-            md = QMimeData()
-            md.setText(self.selected_text)
-            clipboard.setMimeData(md, clipboard.Selection)
+    def createMimeDataFromSelection(self):
+        ans = QMimeData()
+        ans.setText(self.selected_text)
+        return ans
+
+    def show_tooltip(self, ev):
+        pass
+
+    def override_shortcut(self, ev):
+        if iswindows and self.windows_ignore_altgr_shortcut(ev):
+            ev.accept()
+            return True
+
+    def windows_ignore_altgr_shortcut(self, ev):
+        import win32api, win32con
+        s = win32api.GetAsyncKeyState(win32con.VK_RMENU) & 0xffff  # VK_RMENU == R_ALT
+        return s & 0x8000
 
     def event(self, ev):
-        if ev.type() == ev.ShortcutOverride and ev in (QKeySequence.Copy, QKeySequence.Cut):
-            ev.accept()
-            (self.copy if ev == QKeySequence.Copy else self.cut)()
+        et = ev.type()
+        if et == ev.ToolTip:
+            self.show_tooltip(ev)
             return True
+        if et == ev.ShortcutOverride:
+            ret = self.override_shortcut(ev)
+            if ret:
+                return True
         return QPlainTextEdit.event(self, ev)
+
 # }}}
 
+
 if __name__ == '__main__':
     app = QApplication([])
     AddCover.test()
